"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.vuePlugin = exports.vueHotReload = exports.vueComponentNormalizer = exports.vueCache = void 0;
const component_compiler_utils_1 = require("@vue/component-compiler-utils");
const fs = __importStar(require("fs-extra"));
const hash_sum_1 = __importDefault(require("hash-sum"));
const lru_cache_1 = __importDefault(require("lru-cache"));
const esbuildService_1 = require("./esbuildService");
const componentNormalizer_1 = require("./componentNormalizer");
const vueHotReload_1 = require("./vueHotReload");
const path_1 = __importDefault(require("path"));
const serverPluginModuleRewrite_1 = require("vite/dist/server/serverPluginModuleRewrite");
const utils_1 = require("vite/dist/utils");
const vueTemplateCompiler = require('vue-template-compiler');
exports.vueCache = new lru_cache_1.default({
    max: 65535,
});
const defaultExportRE = /((?:^|\n|;)\s*)export default/;
exports.vueComponentNormalizer = '/vite/vueComponentNormalizer';
exports.vueHotReload = '/vite/vueHotReload';
exports.vuePlugin = ({ root, app, resolver, watcher, config, }) => {
    app.use(async (ctx, next) => {
        if (ctx.path === '/vite/hmr') {
            await next();
            ctx.type = 'js';
            ctx.body = ctx.body.replace(/__VUE_HMR_RUNTIME__\.rerender\(path, (.+)\)/g, '__VUE_HMR_RUNTIME__.rerender(path, m)');
            return;
        }
        if (ctx.path === exports.vueHotReload) {
            ctx.type = 'js';
            ctx.body = vueHotReload_1.vueHotReloadCode;
            return;
        }
        if (ctx.path === exports.vueComponentNormalizer) {
            ctx.type = 'js';
            ctx.body = componentNormalizer_1.normalizeComponentCode;
            return;
        }
        if (!ctx.path.endsWith('.vue') && !ctx.vue) {
            return next();
        }
        const query = ctx.query;
        const publicPath = ctx.path;
        let filePath = resolver.requestToFile(publicPath);
        const source = readFile(filePath);
        const descriptor = component_compiler_utils_1.parse({
            source,
            compiler: vueTemplateCompiler,
            filename: filePath,
            sourceRoot: root,
            needMap: true,
        });
        if (!descriptor) {
            return;
        }
        if (!query.type) {
            // rely on vite internal sfc parse....
            await next();
            ctx.type = 'js';
            ctx.body = await parseSFC(root, filePath, publicPath, descriptor, resolver);
            return;
        }
        if (query.type === 'template') {
            const templateBlock = descriptor.template;
            if (templateBlock && templateBlock.src) {
                const srcPath = utils_1.cleanUrl(serverPluginModuleRewrite_1.resolveImport(root, publicPath, templateBlock.src, resolver));
                templateBlock.content = readFile(resolver.requestToFile(srcPath));
            }
            ctx.type = 'js';
            ctx.body = compileSFCTemplate(templateBlock, filePath, publicPath);
            return;
        }
        if (query.type === 'style') {
            return next();
        }
    });
};
function readFile(filePath) {
    return fs.readFileSync(filePath).toString();
}
async function parseSFC(root, filePath, publicPath, descriptor, resolver) {
    const hasFunctional = descriptor.template && descriptor.template.attrs.functional;
    const id = hash_sum_1.default(publicPath);
    // template
    let templateImport = `var render, staticRenderFns`;
    if (descriptor.template) {
        templateImport = `import { render, staticRenderFns } from "${publicPath}?type=template"`;
    }
    // script
    let scriptImport = `var script = {}`;
    if (descriptor.script) {
        const scriptBlock = descriptor.script;
        let code = scriptBlock.content;
        if (scriptBlock && scriptBlock.src) {
            const srcPath = serverPluginModuleRewrite_1.resolveImport(root, publicPath, scriptBlock.src, resolver);
            code = readFile(resolver.requestToFile(srcPath));
        }
        if (scriptBlock.lang === 'ts') {
            code = (await esbuildService_1.transform(code, publicPath, {
                loader: 'ts',
            })).code;
        }
        // rewrite export default.
        // fast path: simple regex replacement to avoid full-blown babel parse.
        let replaced = code.replace(defaultExportRE, '$1var script =');
        // if the script somehow still contains `default export`, it probably has
        // multi-line comments or template strings. fallback to a full parse.
        // todo
        // if (defaultExportRE.test(replaced)) {
        // 	replaced = rewriteDefaultExport(code)
        // }
        scriptImport = replaced;
    }
    let stylesCode = ``;
    let hasScoped;
    if (descriptor.styles.length) {
        descriptor.styles.forEach((s, i) => {
            const styleRequest = publicPath + `?type=style&index=${i}`;
            if (s.scoped)
                hasScoped = true;
            if (s.module) {
                const styleVar = `__style${i}`;
                const moduleName = typeof s.module === 'string' ? s.module : '$style';
                stylesCode += `\nimport ${styleVar} from ${JSON.stringify(styleRequest + '&module')}`;
                stylesCode += `\n__cssModules[${JSON.stringify(moduleName)}] = ${styleVar}`;
            }
            else {
                stylesCode += `\nimport ${JSON.stringify(styleRequest)}`;
            }
        });
    }
    let code = `
${templateImport}
${scriptImport}
const __cssModules = {}
${stylesCode}
/* normalize component */
import normalizer from "${exports.vueComponentNormalizer}"
var component = normalizer(
  script,
  render,
  staticRenderFns,
  ${hasFunctional ? `true` : `false`},
  injectStyles,
  ${hasScoped ? JSON.stringify(id) : `null`},
  null,
  null
)
  `.trim() + `\n`;
    code += `
function injectStyles (context) {
  for(let o in __cssModules){
    this[o] = __cssModules[o]
  }
}  
  `;
    // TODO custom block
    // if (needsHotReload) {
    // 	code += `\n` + genHotReloadCode(id, hasFunctional, templateRequest)
    // }
    // // Expose filename. This is used by the devtools and Vue runtime warnings.
    // if (process.env.NODE_ENV === 'production') {
    // 	// Expose the file's full path in development, so that it can be opened
    // 	// from the devtools.
    // 	code += `\ncomponent.options.__file = ${JSON.stringify(rawShortFilePath.replace(/\\/g, '/'))}`
    // } else if (options.exposeFilename) {
    // 	// Libraries can opt-in to expose their components' filenames in production builds.
    // 	// For security reasons, only expose the file's basename in production.
    // 	code += `\ncomponent.options.__file = ${JSON.stringify(filename)}`
    // }
    code += `
/* hot reload */
import __VUE_HMR_RUNTIME__ from "${exports.vueHotReload}"
import vue from "vue"
if (import.meta.hot) {
	__VUE_HMR_RUNTIME__.install(vue)
	if(__VUE_HMR_RUNTIME__.compatible){
		 if (!__VUE_HMR_RUNTIME__.isRecorded('${publicPath}')) {
			 __VUE_HMR_RUNTIME__.createRecord('${publicPath}', component.options)
		 	 // import.meta.hot.acceptDeps("${publicPath}?type=template", (mod) => {
		 		// 	 component.options = mod.render
		 		// 	 component.options = mode.staticRenderFns
		 		// 	 __VUE_HMR_RUNTIME__.${hasFunctional ? 'rerender' : 'reload'}('${publicPath}', component.options)
		 	 // })
		 }
	} else {
			console.log("The hmr is not compatible.")
	}
}`;
    code += `\nexport default component.exports`;
    return code;
}
function compileSFCTemplate(block, filePath, publicPath) {
    const { tips, errors, code } = component_compiler_utils_1.compileTemplate({
        source: block.content,
        filename: filePath,
        compiler: vueTemplateCompiler,
        // compilerOptions,
        // allow customizing behavior of vue-template-es2015-compiler
        // transpileOptions: options.transpileOptions,
        transformAssetUrls: true,
        transformAssetUrlsOptions: {
            base: path_1.default.posix.dirname(publicPath),
        },
        isProduction: process.env.NODE_ENV === 'production',
        isFunctional: !!block.attrs.functional,
        optimizeSSR: false,
        prettify: false,
    });
    if (tips) {
        tips.forEach(console.warn);
    }
    if (errors) {
        errors.forEach(console.error);
    }
    return code + `\nexport { render, staticRenderFns }`;
}
//# sourceMappingURL=serverPlugin.js.map